#ifndef PICOLED_GFX_2D_H
#define PICOLED_GFX_2D_H

#include "picoled.h"

namespace picoled::gfx
{

class vec2
{
public:
  float x;
  float y;

  vec2 operator+(const vec2& other) const { return { x + other.x, y + other.y }; }
  vec2 operator-(const vec2& other) const { return { x - other.x, y - other.y }; }
  vec2 operator*(const vec2& other) const { return { x * other.x, y * other.y }; }
  vec2 operator/(const vec2& other) const { return { x / other.x, y / other.y }; }

  float operator%(const vec2& other) const { return x * other.x + y * other.y; }
  float operator^(const vec2& other) const { return x * other.y - y * other.x; }
};

class vec3
{
public:
  float x;
  float y;
  float z;

  vec3 operator+(const vec3& other) const { return { x + other.x, y + other.y, z + other.z }; }
  vec3 operator-(const vec3& other) const { return { x - other.x, y - other.y, z - other.z }; }
  vec3 operator*(const vec3& other) const { return { x * other.x, y * other.y, z * other.z }; }
  vec3 operator/(const vec3& other) const { return { x / other.x, y / other.y, z / other.z }; }

  float operator%(const vec3& other) const { return x * other.x + y * other.y + z * other.z; }
  vec3 operator^(const vec3& other) const
  {
    return
    {
      y * other.z - z * other.y,
      z * other.x - x * other.z,
      x * other.y - y * other.x
    };
  }
};

class vertex2
{
public:
  vec2 position;
  vec2 texture_coordinates;
};

class triangle
{
public:
  vertex2 a;
  vertex2 b;
  vertex2 c;

  triangle fix_winding() const;
  vec3 barycentric(const vec2& p) const;
  bool contains(const vec2& p) const;

  vec2 min() const;
  vec2 max() const;

private:
  float area_double() const;
};

class gfx2d
{
public:
  explicit gfx2d(oled& screen) :
    screen(&screen)
  {}

  gfx2d(const gfx2d&) = delete;
  gfx2d(gfx2d&&) = delete;

  gfx2d& operator=(const gfx2d&) = delete;
  gfx2d& operator=(gfx2d&&) = delete;

  void draw_triangle(triangle tri, const buffer<pixel>& texture, bool ignore_winding = true);

private:
  oled* screen;
};

}

#endif//PICOLED_GFX_2D_H
